/*
 * global.c -- implementation of global stuff
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <global.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

int	debug = 0;
int	verbose = 0;
int	outerspace = 6;

section_t	*rocket = NULL;

section_t	*section_first() {
	return rocket;
}

section_t	*section_next(section_t *current) {
	if (NULL != current) return current->next;
	return NULL;
}

section_t	*section_new() {
	section_t	*result = (section_t *)malloc(sizeof(section_t));
	result->type = -1;
	return result;
}

section_t	*section_add(section_t *add) {
	/* find last element of section chain */
	section_t	*last = rocket;
	if (NULL == rocket) {
		if (debug)
			fprintf(stderr, "%s:%d: first element is type %d\n",
				__FILE__, __LINE__, add->type);
		return rocket = add;
	}
	while (NULL != last->next)
		last = last->next;

	/* append the new section */
	if (debug)
		fprintf(stderr, "%s:%d: adding type %d to type %d\n",
			__FILE__, __LINE__, add->type, last->type);
	last->next = add;
	add->next = NULL;
	return add;
}

int	section_compute(section_t *s, double d) {
	return s->compute(s, d);
}

int	section_compute_all() {
	int	rc;
	section_t	*p = section_first();
	double	d;
	if (SECTION_NOSECONE != p->type) {
		fprintf(stderr, "first element must be a nosecone\n");
		exit(EXIT_FAILURE);
	}
	d = 2. * p->detail.nosecone.radius;
	for (; NULL != p; p = section_next(p)) {
		if ((rc = section_compute(p, d)) < 0) {
			return rc;
		}
	}
	return 0;
}

double	section_Cn() {
	section_t	*p;
	double	Cn = 0.;
	if (debug)
		fprintf(stderr, "%s:%d: accumulating Cn\n",
			__FILE__, __LINE__);
	for (p = section_first(); NULL != p; p = section_next(p)) {
		Cn += p->Cn;
	}
	if (debug)
		fprintf(stderr, "%s:%d: total Cn = %f\n",
			__FILE__, __LINE__, Cn);
	return Cn;
}

double	section_Xn() {
	section_t	*p;
	double	Xn = 0.;
	double	l = 0.;
	if (debug)
		fprintf(stderr, "%s:%d: accumulating Xn\n",
			__FILE__, __LINE__);
	for (p = section_first(); NULL != p; p = section_next(p)) {
		Xn += p->Cn * (l + p->deltaX);
		l += p->length;
	}
	if (debug)
		fprintf(stderr, "%s:%d: Xn = %f\n", __FILE__, __LINE__,
			Xn/section_Cn());
	return Xn/section_Cn();
}

param_t	*param_get(char *arg, param_t *params, int n) {
	int	i, l;
	if (debug)
		fprintf(stderr, "%s:%d: looking for parameter %s\n",
			__FILE__, __LINE__, arg);
	if (NULL == arg)
		return NULL;
	for (i = 0; i < n; i++) {
		l = strlen(params[i].name);
		if (0 == strncmp(arg, params[i].name, l)) {
			params[i].value = atof(arg + l);
			if (debug)
				fprintf(stderr, "%s:%d: found %s = %f\n",
					__FILE__, __LINE__, arg, 
					params[i].value);
			return &params[i];
		}
	}
	return NULL;
}

double	sqr(double x) {
	return x * x;
}

void	section_print(section_t *s, FILE *f, int *w) {
	s->print(s, f, w);
}

void	section_print_all(FILE *f) {
	double	length = 0.;
	int	width;
	section_t	*s;
	fprintf(f, "the rocket consists of the following parts:\n");
	width = 0;
	for (s = section_first(); NULL != s; s = section_next(s)) {
		length += s->length;
		section_print(s, f, &width);
	}
	fprintf(f, "total length: %f\n", length);
}
